@@ -2,16 +2,15 @@ import logging
 import re
 import threading
 import os
-from numpy import dtype
+import numpy as np
 import pandas as pd
 import codecs
 import requests
-from tqdm import tqdm
+from tqdm.auto import tqdm
 from bs4 import BeautifulSoup
-from datetime import date, datetime
+from datetime import datetime
 from abc import abstractmethod
 from ekorpkit import eKonf
-from ekorpkit.io.file import save_dataframe, load_dataframe
 
 
 log = logging.getLogger(__name__)
@@ -220,21 +219,18 @@ class FOMC:
 
     def save(self):
         """
-        save the dataframe to a csv file
+        save the dataframe to a parquet file
         """
-        if self.verbose:
-            print("Writing to ", self.output_filepath)
-        os.makedirs(self.output_dir, exist_ok=True)
-        save_dataframe(self.df, self.output_filepath)
+        log.info("Writing to ", self.output_filepath)
+        eKonf.save_data(self.df, self.output_filepath)
 
     def load_calendar(self, from_year=None, force_download=False):
         """
         get the calendar from the FOMC website
         """
         if os.path.exists(self.calendar_filepath) and not force_download:
-            if self.verbose:
-                print("Loading calendar from cache...")
-            self.calendar = load_dataframe(self.calendar_filepath)
+            log.info("Loading calendar from cache...")
+            self.calendar = eKonf.load_data(self.calendar_filepath)
             return self.calendar
 
         if self.verbose:
@@ -355,7 +351,7 @@ class FOMC:
         # # Use date as index
         df.set_index("date", inplace=True)
 
-        save_dataframe(df, self.calendar_filepath)
+        eKonf.save_data(df, self.calendar_filepath)
         self.calendar = df
         return df.copy()
 
@@ -370,7 +366,7 @@ class FOMC:
           +1: Rate hike
         """
         import numpy as np
-        from tqdm import tqdm
+        from tqdm.auto import tqdm
 
         # fedrates = fedrates.copy()[fedrates.index >= self.calendar.index.min()]
         fedrates["rate"] = fedrates[series_id].shift(-3)
@@ -399,7 +395,7 @@ class FOMC:
         fomc_calendar[columns] = results[columns]
         fomc_calendar = fomc_calendar[fomc_calendar["rate"].notnull()]
 
-        save_dataframe(fomc_calendar, self.calendar_filepath)
+        eKonf.save_data(fomc_calendar, self.calendar_filepath)
         self.calendar = fomc_calendar
         return fomc_calendar.copy()
 
@@ -427,7 +423,7 @@ class FOMC:
             lambda x: 0 if x == 0 else 1
         )
 
-        save_dataframe(fomc_calendar, self.calendar_filepath)
+        eKonf.save_data(fomc_calendar, self.calendar_filepath)
         self.calendar = fomc_calendar
         return fomc_calendar.copy()
 
@@ -521,6 +517,13 @@ class FOMC:
             else None,
             axis=1,
         )
+        df["recent_meeting"] = df["date"].map(
+            lambda x: self._get_recent_meeting_date(x)
+        )
+        df["recent_decision"] = df["recent_meeting"].map(
+            lambda x: self._get_rate_change(x)
+        )
+        df["recent_rate"] = df["recent_meeting"].map(lambda x: self._get_rate(x))
         df["next_meeting"] = df["date"].map(lambda x: self._get_next_meeting_date(x))
         df["next_decision"] = df["next_meeting"].map(lambda x: self._get_rate_change(x))
         df["next_rate"] = df["next_meeting"].map(lambda x: self._get_rate(x))
@@ -577,3 +580,105 @@ class FOMC:
                 return dt_ix
             else:
                 return None
+
+    def _get_recent_meeting_date(self, article_date):
+        """
+        Returns the most recent fomc meeting date for the given date x, referring to fomc_calendar DataFrame.
+        Usually FOMC Meetings takes two days, so it starts searching from x+2.
+        x should be of datetime type or yyyy-mm-dd format string.
+        """
+        import datetime as dt
+
+        if isinstance(article_date, str):
+            article_date = datetime.strptime(article_date, "%Y-%m-%d")
+
+        # Add two days to get the day after next
+        article_date += dt.timedelta(days=2)
+
+        cal = self.calendar.copy()
+        if cal.index.min() > article_date:
+            # If the date is older than the first FOMC Meeting, do not return any date.
+            return None
+        else:
+            dt_ix = cal[cal.index < article_date].index.max()
+            if not pd.isnull(dt_ix):
+                return dt_ix
+            else:
+                return None
+
+    @staticmethod
+    def get_irf(data, nlags=4, nirf=20):
+        import statsmodels.api as sm
+
+        # Dimensions
+        nobs = data.shape[0] - nlags
+        neqs = data.shape[1]
+
+        # Estimate via statsmodels VAR
+        mod = sm.tsa.VAR(data)
+        res = mod.fit(maxlags=nlags, ic=None, trend="n")
+
+        # Variance / Covariance matrix
+        omega = res.sigma_u
+
+        # Take LDL decomposition of the variance / covariance matrix
+        L = np.linalg.cholesky(omega)
+        D = np.diag(L.diagonal() ** 2)
+        L[np.diag_indices(L.shape[0])] = 1
+
+        # Storage
+        irf_tilde = np.zeros((nirf + nlags + 1, neqs, neqs))
+        irf = np.zeros((nirf, neqs, neqs))
+
+        # Initial values
+        irf_tilde[nlags] = np.eye(neqs)
+        irf[0] = L
+
+        # Iteratively create IRFs
+        for i in range(nirf):
+            for j in range(nlags):
+                irf_tilde[i + nlags + 1] += np.dot(
+                    res.params.iloc[j * neqs : (j + 1) * neqs, :].T,
+                    irf_tilde[i + nlags - j],
+                )
+            irf[i] = np.dot(irf_tilde[i + nlags], L)
+
+        irfs = {}
+        for col in data.columns:
+            ix = data.columns.get_loc(col)
+            irf_df = pd.DataFrame(-irf[:, :, ix])
+            irf_df.columns = data.columns
+            irfs[col] = irf_df
+
+        return irfs
+
+    @staticmethod
+    def plot_irf(irfs, impulse_name, ncols=2, figsize=(16, 12), title=None):
+        irf = irfs[impulse_name]
+
+        title = title or f"Impulse Response of {impulse_name} shock"
+        names = irf.columns.tolist()
+        neqs = len(names)
+
+        cfg = eKonf.compose("visualize/plot=lineplot")
+
+        cfg.figure.figsize = figsize
+        cfg.subplots.ncols = ncols
+        cfg.subplots.nrows = int(np.ceil(neqs / ncols))
+        lineplot = cfg.lineplot.copy()
+        ax = cfg.ax.copy()
+        cfg.plots = []
+        cfg.axes = []
+
+        for i, name in enumerate(names):
+            plot = lineplot.copy()
+            plot.y = name
+            plot.axno = i
+            cfg.plots.append(plot)
+            _ax = ax.copy()
+            _ax.title = name
+            _ax.axno = i
+            cfg.axes.append(_ax)
+        cfg.figure.super.title = title
+
+        eKonf.instantiate(cfg, data=irf)
